### AUTOGENERATED
import requests
import os
from typing import Union
from pprint import pformat
import logging

logger = logging.getLogger(__name__)

# ExeccutionError is derived from ValueError to keep backward compatibility.
class ExecutionError(ValueError):
	"""Raise if API returns an error code != 400"""

	def __init__(self, status_code, reason, result):
		self.status_code = status_code
		self.reason = reason
		self.result = result
		message = f'Request returned unexpected result ({self.status_code}: {self.reason}). Result: \n{pformat(self.result)}'
		super().__init__(message)


class APIObject(object):
	pass


class ExplicitNull(object):
	pass


class APIEndpoint(object):
	def __init__(self, netdb_base_url:str=None, netdb_token:str=None, **kwargs):
		self.base_url = netdb_base_url if netdb_base_url is not None else kwargs.get('base_url', None)
		self.version = '4.0'
		self.token = netdb_token if netdb_token is not None else kwargs.get('token', None)


class APISession(object):
	def __init__(self, endpoint: APIEndpoint, protocol: str='https', update_check: bool='NETDB_CLIENT_DISABLE_UPDATE_CHECK' not in os.environ):
		self._build_version = '4.0.700'
		self.session = requests.session()
		self.session.headers.update({'Authorization': 'Bearer ' + endpoint.token})
		self.api_root = f'{protocol}://{endpoint.base_url}'
		if 'www-net' in endpoint.base_url:
			self.base_url = f'{self.api_root}/api/{endpoint.version}'
		else:
			self.base_url = f'{self.api_root}/{endpoint.version}'
		try:
			if update_check:
				versions = requests.get(self.api_root).json()[0]
				cur_v = None
				ref = [int(r) for r in endpoint.version.split('.')]
				for v in versions:
					if v['major'] == ref[0] and v['minor'] == ref[1]:
						if self._build_version != v['numeric']:
							logger.warning(f"This library was built for NETDB API version {self._build_version}, but API endpoint reports version {v['numeric']}. "
						                "Please consider updating if you encounter any unexpected behavior. "
						                "(You can disable this message by setting NETDB_CLIENT_DISABLE_UPDATE_CHECK in your environment or by passing update_check=False to APISession)")
						break
		except:
			pass

	def execute_ta(self, ta: list, dry_mode: bool = False, dict_mode: bool = False, su_login_name: str = None) -> Union[list, dict]:
		res = self.session.post(f'{self.base_url}/wapi/transaction/execute?dry_mode={str(dry_mode).lower()}&dict_mode={str(dict_mode).lower()}{"" if su_login_name is None else "&su_login_name="+su_login_name}', json=ta)
		if not res.ok:
			raise ExecutionError(res.status_code, res.reason, res.json() if "json" in res.headers["content-type"] else res.text)
		return res.json()

	def close(self):
		self.session.close()


def _denullify_dict(d):
	d = {k:v for k,v in d.items() if v is not None}
	for k,v in d.items():
		if isinstance(v, ExplicitNull):
			d[k] = None
	return d
